package grmi

import (
	"LEIT_PM/rpc"
	"reflect"
	"strings"
)

type Model struct {
	MoudleName                string
	TableName                 string
	ModelType                 reflect.Type
	ModelPointerType          reflect.Type
	ModelListPointerType      reflect.Type
	BuildCursor               func() interface{}
	BuildList                 func() interface{}
	SingleRecordParameterInfo *ParameterInfo
	MultiRecordsParameterInfo *ParameterInfo
	Indicator                 interface{}
	KeyColumns                []*ColumnField
	AutomaticUpdatingMembers  []*ColumnField
	ColumnFieldMapping        map[string]*ColumnField
	UrlParameterMapping       map[string]*ColumnField
	Keys                      []string
	UpdatingMembers           []string
	Relations                 []*RelationField
}

func NewModel(moudleName string, tableName string, buildCursor func() interface{}, buildList func() interface{}, columnFields []*ColumnField, updatingMembers []string, relations []*RelationField) *Model {

	keyColumns := make([]*ColumnField, 0, 16)
	automaticUpdatingMembers := make([]*ColumnField, 0, 16)
	keys := make([]string, 0, 16)
	urlParameterMapping := make(map[string]*ColumnField, len(columnFields))
	for _, field := range columnFields {
		urlParameterMapping[field.UrlParameterName] = field
		if field.IsKey {
			keys = append(keys, field.ColumnName)
			keyColumns = append(keyColumns, field)
		}
		if field.Source == Session {
			automaticUpdatingMembers = append(automaticUpdatingMembers, field)
		}
	}
	fieldMapping := make(map[string]*ColumnField, len(columnFields))
	for _, field := range columnFields {
		fieldMapping[field.Name] = field
	}
	indicator := buildCursor()
	modelType := reflect.TypeOf(indicator).Elem()
	modelPointerType := reflect.PtrTo(modelType)
	modelListPointerType := reflect.PtrTo(reflect.SliceOf(modelType))
	bodyParser4ModelPointerType, err := rpc.DefaultValueParserBuilder.Build("body", modelPointerType)
	if err != nil {
		panic(err)
	}
	bodyParser4ModelListPointerType, err := rpc.DefaultValueParserBuilder.Build("body", modelListPointerType)
	if err != nil {
		panic(err)
	}
	singleRecordParameterInfo := &ParameterInfo{
		Type:   modelPointerType,
		Parser: BuildParserOfType(bodyParser4ModelPointerType),
	}
	multiRecordsParameterInfo := &ParameterInfo{
		Type:   modelListPointerType,
		Parser: BuildParserOfType(bodyParser4ModelListPointerType),
	}
	return &Model{moudleName, tableName, modelType, modelPointerType, modelListPointerType, buildCursor, buildList, singleRecordParameterInfo, multiRecordsParameterInfo, indicator, keyColumns, []*ColumnField{}, fieldMapping, urlParameterMapping, keys, updatingMembers, relations}
}

func (model *Model) BuildPredicates(urlParameters map[string]string) ([]*Predicate, error) {
	predicates := make([]*Predicate, 0, len(urlParameters))
	for name, value := range urlParameters {
		item, ok := model.UrlParameterMapping[strings.ToLower(name)]
		if !ok {
			continue
		}
		predicateValue, err := item.ValueParser(value)
		if err != nil {
			return nil, NewRequestError(err)
		}
		predicates = append(predicates, item.NewPredicate(item.PredicateType, predicateValue))
	}
	return predicates, nil
}
