package grmi

import (
	"strconv"
)

const (
	PageSize    = "pageSize"
	_PageSize   = "_pageSize"
	PageNumber  = "pageNumber"
	_PageNumber = "_pageNumber"
)

type Paging struct {
	Number int64
	Size   int64
}

func NewPaging(number int64, size int64) *Paging {
	return &Paging{Number: number, Size: size}
}

func (self *Paging) Offset() int64 {
	return (self.Number - 1) * self.Size
}

func getValue(urlParameters map[string]string, names []string, defaultValue int64) int64 {
	for _, name := range names {
		if urlParameter, ok := urlParameters[name]; ok {
			value, err := strconv.ParseInt(urlParameter, 10, 64)
			if err == nil {
				return value
			}
		}
	}
	return defaultValue
}

func getPageIndex(urlParameters map[string]string) int64 {
	return getValue(urlParameters, []string{_PageNumber, PageNumber}, 1)
}

func getPageSize(urlParameters map[string]string) int64 {
	return getValue(urlParameters, []string{_PageSize, PageSize}, 10)
}

func GetPaging(urlParameters map[string]string) *Paging {
	size := getPageSize(urlParameters)
	number := getPageIndex(urlParameters)
	return NewPaging(number, size)
}
