package dal

import (
	"LEIT_PM/grmi"
	"errors"
	"fmt"
	"github.com/go-xorm/xorm"
	"reflect"
	"strings"
	"xorm.io/core"
)

func InsertOne(session *xorm.Session, model *grmi.Model, entity grmi.Entity, entityValue interface{}) error {
	// entityValue 类型检查 实体
	entity.SetLastUser("anonymous")
	_, err := session.Table(model.TableName).Insert(entityValue)
	if err != nil {
		return err
	}
	return nil
}

func InsertMany(session *xorm.Session, model *grmi.Model, entitiesValue interface{}) error {
	modelsValue := reflect.ValueOf(entitiesValue).Elem() // 类型检查 models *[]实体
	for i := 0; i < modelsValue.Len(); i++ {
		entityValue := modelsValue.Index(i).Addr().Interface()
		model := entityValue.(grmi.Entity) // 类型检查 grmi.Entity
		model.SetLastUser("anonymous")
	}
	_, err := session.Table(model.TableName).Insert(entitiesValue)
	if err != nil {
		return err
	}
	return nil
}

func DeleteOne(session *xorm.Session, model *grmi.Model, pks core.PK) error {
	_, err := session.Table(model.TableName).ID(pks).Delete(model.Indicator)
	if err != nil {
		return err
	}
	return nil
}

func DeleteMany(session *xorm.Session, model *grmi.Model, entitiesValue interface{}) error {
	modelsValue := reflect.ValueOf(entitiesValue).Elem() // 类型检查 models *[]实体
	for i := 0; i < modelsValue.Len(); i++ {
		entityValue := modelsValue.Index(i).Addr().Interface()
		entity := entityValue.(grmi.Entity) // 类型检查 grmi.Entity
		_, err := session.Table(model.TableName).Cols(model.UpdatingMembers...).ID(entity.GetKey()).Delete(model.Indicator)
		if err != nil {
			return err
		}
	}
	return nil
}

func DeleteWhere(session *xorm.Session, model *grmi.Model, predicates []grmi.Predicate) error {

	var parameters []interface{}
	where := "1 = 1"

	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	_, err := session.Delete(model.Indicator)
	if err != nil {
		return err
	}
	return nil
}

func SelectOne(session *xorm.Session, model *grmi.Model, pks core.PK, withRelations bool) (interface{}, error) {
	data := model.BuildCursor()
	ok, err := session.Table(model.TableName).ID(pks).Get(data)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, nil
	}
	if withRelations {
		moudleMapping, ok := grmi.Models()[model.MoudleName]
		if !ok {
			return nil, errors.New(fmt.Sprintf("未找到模块:%s!", model.MoudleName))
		}
		for _, relation := range model.Relations {
			relationEntity, ok := moudleMapping[strings.ToLower(relation.ModelName)]
			if !ok {
				return nil, errors.New(fmt.Sprintf("未找到关联实体%s!", relation.ModelName))
			}

			relationData, err := selectRelations(session, data, pks, model, relationEntity, relation)
			if err != nil {
				return nil, err
			}

			valueOfData := reflect.ValueOf(data)
			field := valueOfData.Elem().FieldByName(relation.Name)
			field.Set(reflect.ValueOf(relationData))
		}
	}
	return data, nil
}

func selectRelations(session *xorm.Session, data interface{}, pks core.PK, model *grmi.Model, relationEntity *grmi.Model, relation *grmi.RelationField) (interface{}, error) {
	session = session.Table(relationEntity.TableName)
	session = session.Join("INNER", model.TableName, relation.On)
	if relation.Many {
		if len(pks) != len(model.KeyColumns) {
			return nil, errors.New(fmt.Sprintf("主键值的数量与定义不匹配!"))
		}
		for i, key := range model.KeyColumns {
			session = session.Where(fmt.Sprintf("%s = ?", key.FullName), pks[i])
		}
		if relationEntity.Keys != nil {
			for _, key := range relationEntity.KeyColumns {
				session = session.OrderBy(key.FullName)
			}
		}
		relationData := relationEntity.BuildList()
		err := session.Find(relationData)
		if err != nil {
			return nil, err
		}
		return reflect.ValueOf(relationData).Elem().Interface(), nil
	} else {
		pks = make(core.PK, len(relationEntity.KeyColumns))
		valueOfData := reflect.ValueOf(data).Elem()
		for i, key := range relationEntity.KeyColumns {
			for _, columnFields := range relation.Condition {
				if columnFields[1].ColumnName == key.ColumnName {
					field := valueOfData.FieldByName(columnFields[0].Name)
					pks[i] = field.Interface()
					break
				}
			}
		}
		session = session.ID(pks)
		relationData := relationEntity.BuildCursor()
		ok, err := session.Get(relationData)
		if err != nil {
			return nil, err
		}
		if !ok {
			return nil, nil
		}
		return relationData, nil
	}
}

func SelectMany(session *xorm.Session, model *grmi.Model, predicates []*grmi.Predicate, orderByFields []string) (interface{}, error) {

	var parameters []interface{}
	where := "1=1 "

	session = session.Table(model.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	if orderByFields != nil {
		for _, field := range orderByFields {
			session = session.OrderBy(field)
		}
	}

	data := model.BuildList()
	err := session.Find(data)
	if err != nil {
		return nil, err
	}
	return reflect.ValueOf(data).Elem().Interface(), nil
}

func SelectManyAndPaging(session *xorm.Session, model *grmi.Model, paging *grmi.Paging, predicates []*grmi.Predicate, orderByFields []string) (*grmi.PagingResult, error) {

	var parameters []interface{}
	where := "1=1 "

	session4Count := session.Table(model.TableName)
	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session4Count = session4Count.Where(where, parameters...)
	} else {
		session4Count = session4Count.Where(where, parameters...)
	}

	count, err := session4Count.Count(model.Indicator)
	if err != nil {
		return nil, err
	}

	orderBy := " order by " + strings.Join(orderByFields, ",")
	parameters = append(parameters, paging.Offset(), paging.Size)
	where = " where " + where
	data := model.BuildList()
	err = session.Table(model.TableName).SQL("select * from "+model.TableName+where+orderBy+" offset ? row fetch next ? row only", parameters...).Find(data)
	if err != nil {
		return nil, err
	}
	return &grmi.PagingResult{Records: reflect.ValueOf(data).Elem().Interface(), Count: count, PageNumber: paging.Number, PageSize: paging.Size}, nil
}

func UpdateOne(session *xorm.Session, model *grmi.Model, entity grmi.Entity, entityValue interface{}) error {
	// entityValue 类型检查 实体
	entity.SetLastUser("anonymous")
	_, err := session.Table(model.TableName).Cols(model.UpdatingMembers...).ID(entity.GetKey()).Update(entityValue)
	if err != nil {
		return err
	}
	return nil
}

func UpdateMany(session *xorm.Session, model *grmi.Model, entitiesValue interface{}) error {
	modelsValue := reflect.ValueOf(entitiesValue).Elem() // 类型检查 models *[]实体
	for i := 0; i < modelsValue.Len(); i++ {
		entityValue := modelsValue.Index(i).Addr().Interface()
		entity := entityValue.(grmi.Entity) // 类型检查 grmi.Model
		entity.SetLastUser("anonymous")
		_, err := session.Table(model.TableName).Cols(model.UpdatingMembers...).ID(entity.GetKey()).Update(entityValue)
		if err != nil {
			return err
		}
	}
	return nil
}

func UpdateWhere(session *xorm.Session, model *grmi.Model, entity grmi.Entity, entityValue interface{}, predicates []grmi.Predicate, columns ...string) error {

	var parameters []interface{}
	where := "1 = 1"

	if predicates != nil {
		for _, predicate := range predicates {
			where += predicate.Build()
			parameters = append(parameters, predicate.Values...)
		}
		session = session.Where(where, parameters...)
	} else {
		session = session.Where(where, parameters...)
	}

	for _, automaticUpdatingMember := range model.AutomaticUpdatingMembers {
		columns = append(columns, automaticUpdatingMember.ColumnName)
	}
	entity.SetLastUser("anonymous")
	_, err := session.Table(model.TableName).Cols(columns...).Update(entityValue)
	if err != nil {
		return err
	}
	return nil
}
