package main

import (
	"LEIT_PM/configuration"
	"LEIT_PM/container"
	"LEIT_PM/db"
	"LEIT_PM/rpc"
	"LEIT_PM/utilities"
	"fmt"
	_ "github.com/denisenkom/go-mssqldb"
	"github.com/kardianos/service"
	"github.com/kataras/iris/v12"
	"log"
	"os"
	"path/filepath"
)

type Program struct {
}

func (program *Program) Start(_ service.Service) error {
	fmt.Println("MES Server")
	go program.run()
	return nil
}

func newMES(application *iris.Application, config *configuration.RootConfiguration) {

	utilities.PreSettring(application)
	db.Config = config
	engine, err := db.Engine()
	if err != nil {
		panic(err.Error())
	}
	transactionHandlerFactory := container.NewTransactionHandlerFactory(engine)
	container.RegisterRoutes(application, rpc.DefaultMethodInvokerBuilder.Build, container.GlobalInformations, transactionHandlerFactory)
}

func (program *Program) run() {

	defaultLogFilePath := "./mes.log"
	defaultLogFile, err := os.Create(defaultLogFilePath) // 临时日志
	if err != nil {
		log.Println(err)
		return
	}
	defer utilities.Close(defaultLogFile)

	log.SetOutput(defaultLogFile)
	log.SetPrefix("[Debug]")
	log.SetFlags(log.LstdFlags)

	app := iris.New()

	var config configuration.RootConfiguration

	if err := utilities.GetConfigFromJsonFile("./config.json", &config); err != nil {
		log.Println(err)
		return
	}

	defaultLogFilePath, err = filepath.Abs(defaultLogFilePath)
	if err != nil {
		log.Println(err)
		return
	}

	logFilePath, err := filepath.Abs(config.Log.FilePath)
	if err != nil {
		log.Println(err)
		return
	}

	if defaultLogFilePath != logFilePath {
		logFile, err := os.Create(config.Log.FilePath)
		if err != nil {
			log.Println(err)
			return
		}
		defer utilities.Close(logFile)
		log.SetOutput(logFile)
	}

	newMES(app, &config)

	if err := app.Run(iris.Addr(fmt.Sprint("0.0.0.0:", config.Server.Port))); err != nil {
		log.Println(err)
	}
}

func (program *Program) Stop(_ service.Service) error {

	return nil
}

func getServiceName() string {

	return "MES"
}

func main() {

	executable, err := os.Executable()
	if err != nil {
		log.Println(err)
		return
	}
	os.Chdir(filepath.Dir(executable))

	serviceName := getServiceName()
	svrConfig := &service.Config{
		Name:        serviceName,
		DisplayName: serviceName,
		Description: serviceName,
	}
	if err := utilities.LaunchService(new(Program), svrConfig); err != nil {
		log.Println(err)
	}
}
